<?php
/**
 * Voicer converts text into human-like speech in more than 200 voices across 30+ languages and variants.
 * Exclusively on Envato Market: https://1.envato.market/voicer
 *
 * @encoding        UTF-8
 * @version         2.0.2
 * @copyright       Copyright (C) 2018 - 2020 Merkulove (https://merkulov.design/). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Voicer;

use Google\ApiCore\ApiException;
use Google\ApiCore\ValidationException;
use Google\Cloud\TextToSpeech\V1\TextToSpeechClient;
use Merkulove\Voicer;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to implement plugin settings.
 *
 * @since 2.0.0
 * @author Alexandr Khmelnytsky (info@alexander.khmelnitskiy.ua)
 **/
final class Settings {

	/**
	 * Voicer Plugin settings.
	 *
	 * @var array()
	 * @since 2.0.0
	 **/
	public $options = [];

	/**
	 * The one true Settings.
	 *
	 * @var Settings
	 * @since 2.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new Settings instance.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Get plugin settings. */
		$this->get_options();

	}

	/**
	 * Render Tabs Headers.
	 *
	 * @param string $current - Selected tab key.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_tabs( $current = 'general' ) {

		/** Tabs array. */
		$tabs          = [];
		$tabs['general'] = [
			'icon' => 'tune',
			'name' => esc_html__( 'General', 'voicer' )
		];

		/** Show this tabs only if we have key file. */
		if ( $this->options['api-key-file'] ) {
			$tabs['design'] = [
				'icon' => 'brush',
				'name' => esc_html__( 'Design', 'voicer' )
			];

			$tabs['assignments'] = [
				'icon' => 'flag',
				'name' => esc_html__( 'Assignments', 'voicer' )
			];
		}

		/** Activation tab enable only if plugin have Envato ID. */
		$plugin_id = EnvatoItem::get_instance()->get_id();
		if ( (int)$plugin_id > 0 ) {
			$tabs['activation'] = [
				'icon' => 'vpn_key',
				'name' => esc_html__( 'Activation', 'voicer' )
			];
        }

		$tabs['status'] = [
			'icon' => 'info',
			'name' => esc_html__( 'Status', 'voicer' )
		];

		$tabs['uninstall'] = [
			'icon' => 'delete_sweep',
			'name' => esc_html__( 'Uninstall', 'voicer' )
		];

		/** Render Tabs. */
		?>
        <aside class="mdc-drawer">
            <div class="mdc-drawer__content">
                <nav class="mdc-list">

                    <div class="mdc-drawer__header mdc-plugin-fixed">
                        <!--suppress HtmlUnknownAnchorTarget -->
                        <a class="mdc-list-item mdp-plugin-title" href="#wpwrap">
                            <i class="mdc-list-item__graphic" aria-hidden="true">
                                <img src="<?php echo esc_attr( Voicer::$url . 'images/logo-color.svg' ); ?>" alt="<?php echo esc_html__( 'Voicer', 'voicer' ) ?>">
                            </i>
                            <span class="mdc-list-item__text">
                                <?php echo esc_html__( 'Voicer', 'voicer' ) ?>
                                <sup><?php echo esc_html__( 'ver.', 'voicer' ) . esc_html( Voicer::$version ); ?></sup>
                            </span>
                        </a>
                        <button type="submit" name="submit" id="submit"
                                class="mdc-button mdc-button--dense mdc-button--raised">
                            <span class="mdc-button__label"><?php echo esc_html__( 'Save changes', 'voicer' ) ?></span>
                        </button>
                    </div>

                    <hr class="mdc-plugin-menu">
                    <hr class="mdc-list-divider">
                    <h6 class="mdc-list-group__subheader"><?php echo esc_html__( 'Plugin settings', 'voicer' ) ?></h6>

					<?php

					// Plugin settings tabs
					foreach ( $tabs as $tab => $value ) {
						$class = ( $tab == $current ) ? ' mdc-list-item--activated' : '';
						echo "<a class='mdc-list-item " . $class . "' href='?page=mdp_voicer_settings&tab=" . $tab . "'><i class='material-icons mdc-list-item__graphic' aria-hidden='true'>" . $value['icon'] . "</i><span class='mdc-list-item__text'>" . $value['name'] . "</span></a>";
					}

					/** Helpful links. */
					$this->support_link();

					/** Activation Status. */
					PluginActivation::get_instance()->display_status();

					?>
                </nav>
            </div>
        </aside>
		<?php
	}

	/**
	 * Displays useful links for an activated and non-activated plugin.
	 *
	 * @since 2.0.0
     *
     * @return void
	 **/
	public function support_link() { ?>

        <hr class="mdc-list-divider">
        <h6 class="mdc-list-group__subheader"><?php echo esc_html__( 'Helpful links', 'voicer' ) ?></h6>

        <a class="mdc-list-item" href="https://docs.merkulov.design/tag/voicer/" target="_blank">
            <i class="material-icons mdc-list-item__graphic" aria-hidden="true">collections_bookmark</i>
            <span class="mdc-list-item__text"><?php echo esc_html__( 'Documentation', 'voicer' ) ?></span>
        </a>

		<?php if ( PluginActivation::get_instance()->is_activated() ) : /** Activated. */ ?>
            <a class="mdc-list-item" href="https://1.envato.market/voicer-support" target="_blank">
                <i class="material-icons mdc-list-item__graphic" aria-hidden="true">mail</i>
                <span class="mdc-list-item__text"><?php echo esc_html__( 'Get help', 'voicer' ) ?></span>
            </a>
            <a class="mdc-list-item" href="https://1.envato.market/cc-downloads" target="_blank">
                <i class="material-icons mdc-list-item__graphic" aria-hidden="true">thumb_up</i>
                <span class="mdc-list-item__text"><?php echo esc_html__( 'Rate this plugin', 'voicer' ) ?></span>
            </a>
		<?php endif; ?>

        <a class="mdc-list-item" href="https://1.envato.market/cc-merkulove" target="_blank">
            <i class="material-icons mdc-list-item__graphic" aria-hidden="true">store</i>
            <span class="mdc-list-item__text"><?php echo esc_html__( 'More plugins', 'voicer' ) ?></span>
        </a>
		<?php

	}

	/** @noinspection PhpUnused */
	/**
	 * Render settings audio profile field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_audio_profile() {

	    /** Prepare options for select. */
		$options = [
			'wearable-class-device' => esc_html__( 'Smart watches and other wearables', 'voicer' ),
			'handset-class-device' => esc_html__( 'Smartphones', 'voicer' ),
			'headphone-class-device' => esc_html__( 'Earbuds or headphones', 'voicer' ),
			'small-bluetooth-speaker-class-device' => esc_html__( 'Small home speakers', 'voicer' ),
			'medium-bluetooth-speaker-class-device' => esc_html__( 'Smart home speakers', 'voicer' ),
			'large-home-entertainment-class-device' => esc_html__( 'Home entertainment systems', 'voicer' ),
			'large-automotive-class-device' => esc_html__( 'Car speakers', 'voicer' ),
			'telephony-class-application' => esc_html__( 'Interactive Voice Response', 'voicer' ),
		];

		/** Render select. */
		UI::get_instance()->render_select(
			$options,
			$this->options['audio-profile'], // Selected option.
			esc_html__( 'Audio Profile', 'voicer' ),
			esc_html__( 'Optimize the synthetic speech for playback on different types of hardware.', 'voicer' ),
			['name' => 'mdp_voicer_settings[audio-profile]']
		);
    }

	/** @noinspection PhpUnused */
	/**
	 * Render Speaking Rate/Speed field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_speaking_rate() {

		/** Render slider. */
		UI::get_instance()->render_slider(
			$this->options['speaking-rate'],
			0.25,
			4.0,
			0.25,
			esc_html__( 'Speaking Rate/Speed', 'voicer'),
			esc_html__( 'Speaking rate:', 'voicer') . ' <strong>' . esc_html( $this->options['speaking-rate'] ) . '</strong><br>',
			[
				'name' => 'mdp_voicer_settings[speaking-rate]',
				'class' => 'mdc-slider-width',
				'id' => 'mdp_voicer_settings_rate'
			],
			false
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Pitch field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_pitch() {

		/** Render slider. */
		UI::get_instance()->render_slider(
			$this->options['pitch'],
			-20,
			20,
			0.1,
			esc_html__( 'Pitch', 'voicer' ),
			esc_html__( 'Current pitch:', 'voicer') . ' <strong>' . esc_html( $this->options['pitch'] ) . '</strong>',
			[
				'name' => 'mdp_voicer_settings[pitch]',
				'class' => 'mdc-slider-width',
				'id' => 'mdp_voicer_settings_pitch'
			],
			false
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render settings current_language field.
	 *
	 * @throws ApiException
	 * @throws ValidationException
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_language() {
		/** Create client object. */
		$client = new TextToSpeechClient();

		/** Perform list voices request. */
		$response = $client->listVoices();
		$voices   = $response->getVoices();

		if ( count( $voices ) == 0 ) {
			?>
            <div class="mdp-key-error"><?php esc_html_e( 'Failed to get the list of languages. The request failed. It looks like a problem with your API Key File. Make sure that you are using the correct key file, and that the quotas have not been exceeded. If you set security restrictions on a key, make sure that the current domain is added to the exceptions.', 'voicer' ); ?></div><?php
			return;
		}

		/** Prepare Languages Options. */
		$options = [];
		$options[] = esc_html__( 'Select Language', 'voicer' );
        foreach ( $voices as $voice ) {
        	$lang = $this->get_lang_by_code( $voice->getLanguageCodes() );
	        $options["{$lang}"] = $lang;
        }

		/** Render Language select. */
		UI::get_instance()->render_select(
			$options,
			'',
			esc_html__('Language', 'voicer' ),
			'',
			[
                'name' => 'mdp_voicer_language_filter',
				'id' => 'mdp-voicer-language-filter'
            ]
		);

		?>

        <div class="mdc-text-field-helper-line mdp-voicer-helper-padding">
            <div class="mdc-text-field-helper-text mdc-text-field-helper-text--persistent"><?php esc_html_e( 'The list includes both standard and', 'voicer' ); ?>
                <a href="https://cloud.google.com/text-to-speech/docs/wavenet"
                   target="_blank"><?php esc_html_e( 'WaveNet voices', 'voicer' ); ?></a>.
				<?php esc_html_e( 'WaveNet voices are higher quality voices with different', 'voicer' ); ?>
                <a href="https://cloud.google.com/text-to-speech/pricing"
                   target="_blank"><?php esc_html_e( 'pricing', 'voicer' ); ?></a>;
				<?php esc_html_e( 'in the list, they have the voice type "WaveNet".', 'voicer' ); ?>
            </div>
        </div>

        <table id="mdp-voicer-settings-language-tbl" class="display stripe hidden">
            <thead>
            <tr>
                <th><?php esc_html_e( 'Language', 'voicer' ); ?></th>
                <th><?php esc_html_e( 'Voice', 'voicer' ); ?></th>
                <th><?php esc_html_e( 'Gender', 'voicer' ); ?></th>
            </tr>
            </thead>
            <tbody>
			<?php foreach ( $voices as $voice ) : ?>
                <tr <?php if ( $voice->getName() == $this->options['language'] ) { echo 'class="selected"'; } ?>>
                    <td class="mdp-lang-name">
						<?php echo esc_html( $this->get_lang_by_code( $voice->getLanguageCodes() ) ); // Language. ?>
                    </td>
                    <td>
                        <span class="mdp-lang-code" title="<?php echo esc_html( $voice->getLanguageCodes()[0] ); // Language Code ?>"><?php echo esc_html( $voice->getLanguageCodes()[0] ); // Language Code  ?></span>
                        -
                        <span><?php echo $this->get_voice_type( $voice->getName() ); //Voice Type ?></span>
                        -
                        <span class="mdp-voice-name" title="<?php echo esc_html( $voice->getName() ); // Voice name ?>"><?php echo esc_html( substr( $voice->getName(), -1 ) ); // Voice Variant ?></span>
                    </td>
                    <td>
						<?php
						/** SSML voice gender values from TextToSpeech\V1\SsmlVoiceGender. */
						$ssmlVoiceGender = [ 'SSML_VOICE_GENDER_UNSPECIFIED', 'Male', 'Female', 'Neutral' ];

						echo '<span title="' . esc_attr( $ssmlVoiceGender[ $voice->getSsmlGender() ], 'voicer' ) . '"><img src="' . Voicer::$url . 'images/' . strtolower( $ssmlVoiceGender[ $voice->getSsmlGender() ] ) . '.svg" alt="' . esc_attr( $ssmlVoiceGender[ $voice->getSsmlGender() ], 'voicer' ) . '">' . esc_html( $ssmlVoiceGender[ $voice->getSsmlGender() ] ) . '</span>'; ?>
                    </td>
                </tr>
			<?php endforeach;

			$client->close();

			?>
            </tbody>

        </table>

        <input id="mdp-voicer-settings-language" type='hidden' name='mdp_voicer_settings[language]'
               value='<?php echo esc_attr( $this->options['language'] ); ?>'>
        <input id="mdp-voicer-settings-language-code" type='hidden' name='mdp_voicer_settings[language-code]'
               value='<?php echo esc_attr( $this->options['language-code'] ); ?>'>
		<?php
    }

	/**
	 * Return Voice Type.
	 *
	 * @param $lang_name - Google voice name.
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 **/
	public function get_voice_type( $lang_name ) {

		$pos = strpos( $lang_name, 'Wavenet' );

		if ( false === $pos ) {
			return '<img src="' . Voicer::$url . 'images/standard.svg" alt="Standard">' . esc_html( 'Standard' );
		} else {
			return '<img src="' . Voicer::$url . 'images/wavenet.svg" alt="WaveNet">' . esc_html( 'WaveNet' );
		}

	}

	/**
	 * Return language name by code.
	 *
	 * @param $lang_code - Google language code.
	 *
	 * @return mixed
	 * @since 2.0.0
	 * @access public
	 */
	public function get_lang_by_code( $lang_code ) {

		if ( is_object( $lang_code ) ) {
			$lang_code = $lang_code[0];
		}

		/** @noinspection SpellCheckingInspection */
		$languages = [
			'da-DK'  => 'Danish (Dansk)',
			'nl-NL'  => 'Dutch (Nederlands)',
			'en-AU'  => 'English (Australian)',
			'en-GB'  => 'English (UK)',
			'en-US'  => 'English (US)',
			'fr-CA'  => 'French Canada (Français)',
			'fr-FR'  => 'French France (Français)',
			'de-DE'  => 'German (Deutsch)',
			'it-IT'  => 'Italian (Italiano)',
			'ja-JP'  => 'Japanese (日本語)',
			'ko-KR'  => 'Korean (한국어)',
			'nb-NO'  => 'Norwegian (Norsk)',
			'pl-PL'  => 'Polish (Polski)',
			'pt-BR'  => 'Portuguese Brazil (Português)',
			'pt-PT'  => 'Portuguese Portugal (Portugal)',
			'ru-RU'  => 'Russian (Русский)',
			'sk-SK'  => 'Slovak (Slovenčina)',
			'es-ES'  => 'Spanish (Español)',
			'sv-SE'  => 'Swedish (Svenska)',
			'tr-TR'  => 'Turkish (Türkçe)',
			'uk-UA'  => 'Ukrainian (Українська)',
			'ar-XA'  => 'Arabic (العربية)',
			'cs-CZ'  => 'Czech (Čeština)',
			'el-GR'  => 'Greek (Ελληνικά)',
			'en-IN'  => 'Indian English',
			'fi-FI'  => 'Finnish (Suomi)',
			'vi-VN'  => 'Vietnamese (Tiếng Việt)',
			'id-ID'  => 'Indonesian (Bahasa Indonesia)',
			'fil-PH' => 'Philippines (Filipino)',
			'hi-IN'  => 'Hindi (हिन्दी)',
			'hu-HU'  => 'Hungarian (Magyar)',
			'cmn-CN' => 'Chinese (官话)',
            'cmn-TW' => 'Taiwanese Mandarin (中文(台灣))'
		];

		return $languages[ $lang_code ];
	}

	/** @noinspection PhpUnused */
	/**
	 * Render API Key field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_api_key() {

		if ( $this->options['api-key-file'] ) {
			$key_path = get_attached_file( $this->options['api-key-file'] );
			$key_name = basename( $key_path );
		}

		?>
        <div class="mdp-upload">
            <div>
                <input type="hidden" name="mdp_voicer_settings[api-key-file]"
                       id="mdp_voicer_settings[api-key-file]"
                       value="<?php echo esc_attr( $this->options['api-key-file'] ); ?>"/>
                <button class="mdp-select-key-file-btn mdc-button mdc-button--outlined"><?php esc_html_e( 'Select Key File', 'voicer' ); ?></button>
                <div class="mdp-key-file-name">
					<?php if ( isset( $key_name ) ) : ?>
						<?php echo esc_html( $key_name ); ?>
					<?php endif; ?>
                </div>
            </div>
        </div>

		<?php
	}

	/** @noinspection PhpUnused */
	/**
	 * Render settings current_language field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
    public function render_current_language() {
	    ?>
        <div class="mdp-now-used">
            <div>
                <strong><?php echo esc_attr( $this->options['language'] ); ?></strong>
            </div>
            <div>
                <audio controls="">
                    <source src="https://cloud.google.com/text-to-speech/docs/audio/<?php esc_attr_e( Settings::$instance->options['language'] ); ?>.mp3" type="audio/mp3">
				    <?php esc_html_e( 'Your browser does not support the audio element.', 'voicer' ); ?>
                </audio>
            </div>
        </div>
	    <?php
    }

	/** @noinspection PhpUnused */
	/**
	 * Render Tooltip Background Color field.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function render_bgcolor() {

		/** Render colorpicker. */
		UI::get_instance()->render_colorpicker(
			$this->options['bgcolor'],
			esc_html__( 'Background Color', 'voicer' ),
			esc_html__( 'Select tooltip background-color', 'voicer' ),
			[
				'name' => 'mdp_voicer_design_settings[bgcolor]',
				'id' => 'mdp-voicer-bgcolor',
				'readonly' => 'readonly'
			]
		);

	}

	/** @noinspection PhpUnused */
	/**
	 * Render Icon Color field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_iconcolor() {

		/** Render colorpicker. */
		UI::get_instance()->render_colorpicker(
			$this->options['iconcolor'],
			esc_html__( 'Icon Color', 'voicer' ),
			esc_html__( 'Select icon color', 'voicer' ),
			[
				'name' => 'mdp_voicer_design_settings[iconcolor]',
				'id' => 'mdp-voicer-iconcolor',
				'readonly' => 'readonly'
			]
		);

	}

	/**
	 * Add plugin settings page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function add_settings_page() {

		add_action( 'admin_menu', [ $this, 'add_admin_menu' ] );
		add_action( 'admin_init', [ $this, 'settings_init' ] );

	}

	/**
	 * Call render_$key for each field.
	 *
	 * @param $group_name - Settings group name.
	 * @param $option_name - Settings option name.
	 * @param $section_id - Settings section id.
	 * @param array $fields - Settings fields [ key => label ] to render.
	 *
	 * @since 2.0.0
	 * @access public
	 */
	public function call_render_handlers( $group_name, $option_name, $section_id, $fields ) {

	    /** Create settings section. */
		register_setting( $group_name, $option_name );
		add_settings_section( $section_id, '', null, $group_name );

		/** Call render_$key for each field. */
		foreach ( $fields as $key => $label ) {

			/** Skip if render method not exist. */
			if ( ! is_callable( [$this, 'render_' . $key] ) ) { continue; }

			add_settings_field( $key, $label, [$this, 'render_' . $key ], $group_name, $section_id );
		}

    }

	/**
	 * Create Design Tab.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function design_tab() {

		/** Design Tab. */
		$group_name = 'DesignOptionsGroup';
		$section_id = 'mdp_voicer_settings_page_design_section';

		/** Settings fields on general tab. */
		$fields = [];

        $fields['bgcolor'] = esc_html__( 'Background Color:', 'voicer' );
        $fields['iconcolor'] = esc_html__( 'Icon Color:', 'voicer' );

		/** Call render_$key for each field. */
		$this->call_render_handlers( $group_name, 'mdp_voicer_design_settings', $section_id, $fields );

	}

	/**
	 * Create General Tab.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
    public function general_tab() {

	    /** General Tab. */
	    $group_name = 'VoicerOptionsGroup';
	    $section_id = 'mdp_voicer_settings_page_general_section';

	    /** Settings fields on general tab. */
	    $fields = [];

	    /** This settings available only after api key */
	    if ( Settings::$instance->options['api-key-file'] ) {
		    $fields['current_language'] = esc_html__( 'Now used:', 'voicer' );
		    $fields['language'] = esc_html__( 'Select Language:', 'voicer' );
		    $fields['audio_profile'] = esc_html__( 'Audio Profile:', 'voicer' );
		    $fields['speaking_rate'] = esc_html__( 'Speaking Speed:', 'voicer' );
		    $fields['pitch'] = esc_html__( 'Pitch:', 'voicer' );
        }

	    $fields ['api_key'] = esc_html__( 'API Key File:', 'voicer' );

	    /** Call render_$key for each field. */
	    $this->call_render_handlers( $group_name, 'mdp_voicer_settings', $section_id, $fields );

    }

	/**
	 * Generate Settings Page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function settings_init() {

		/** General Tab. */
	    $this->general_tab();

		/** This tab available only after api key. */
		if ( Settings::$instance->options['api-key-file'] ) {

			/** Design Tab. */
			$this->design_tab();

			/** Assignments Tab. */
			AssignmentsTab::get_instance()->add_settings();

        }

		/** Activation Tab. */
		PluginActivation::get_instance()->add_settings();

		/** Status Tab. */
		StatusTab::get_instance()->add_settings();

		/** Uninstall Tab. */
		UninstallTab::get_instance()->add_settings();

	}

	/**
	 * Add admin menu for plugin settings.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function add_admin_menu() {

		add_menu_page(
			esc_html__( 'Voicer Settings', 'voicer' ),
			esc_html__( 'Voicer', 'voicer' ),
			'manage_options',
			'mdp_voicer_settings',
			[ $this, 'options_page' ],
			'data:image/svg+xml;base64,' . base64_encode( file_get_contents( Voicer::$path . 'images/logo-menu.svg' ) ),
			'58.5994'// Always change digits after "." for different plugins.
		);

	}

	/**
	 * Plugin Settings Page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function options_page() {

		/** User rights check. */
		if ( ! current_user_can( 'manage_options' ) ) { return; } ?>

        <!--suppress HtmlUnknownTarget -->
        <form action='options.php' method='post'>
            <div class="wrap">

				<?php
				$tab = 'general';
				if ( isset ( $_GET['tab'] ) ) { $tab = $_GET['tab']; }

				/** Render "Voicer settings saved!" message. */
				$this->render_nags();

				/** Render Tabs Headers. */
				?><section class="mdp-aside"><?php $this->render_tabs( $tab ); ?></section><?php

				/** Render Tabs Body. */
				?><section class="mdp-tab-content"><?php

					/** General Tab. */
					if ( $tab == 'general' ) {
						echo '<h3>' . esc_html__( 'Voicer Settings', 'voicer' ) . '</h3>';
						settings_fields( 'VoicerOptionsGroup' );
						do_settings_sections( 'VoicerOptionsGroup' );

                    /** Design Tab. */
					} elseif ( $tab == 'design' ) {
						echo '<h3>' . esc_html__( 'Design Settings', 'voicer' ) . '</h3>';
						settings_fields( 'DesignOptionsGroup' );
						do_settings_sections( 'DesignOptionsGroup' );

                    /** Assignments Tab. */
					} elseif ( $tab == 'assignments' ) {
						echo '<h3>' . esc_html__( 'Assignments Settings', 'voicer' ) . '</h3>';
						settings_fields( 'VoicerAssignmentsOptionsGroup' );
						do_settings_sections( 'VoicerAssignmentsOptionsGroup' );
						AssignmentsTab::get_instance()->render_assignments();

                    /** Activation Tab. */
					} elseif ( $tab == 'activation' ) {
						settings_fields( 'VoicerActivationOptionsGroup' );
						do_settings_sections( 'VoicerActivationOptionsGroup' );
						PluginActivation::get_instance()->render_pid();

                    /** Status tab. */
					} elseif ( $tab == 'status' ) {
						echo '<h3>' . esc_html__( 'System Requirements', 'voicer' ) . '</h3>';
						StatusTab::get_instance()->render_form();

                    /** Uninstall Tab. */
					} elseif ( $tab == 'uninstall' ) {
						echo '<h3>' . esc_html__( 'Uninstall Settings', 'voicer' ) . '</h3>';
						UninstallTab::get_instance()->render_form();
					}

					?>
                </section>
            </div>
        </form>

		<?php
	}

	/**
	 * Render "Settings Saved" nags.
	 *
	 * @since 2.0.0
     *
     * @return void
	 **/
	public function render_nags() {

		if ( ! isset( $_GET['settings-updated'] ) ) { return; }

		if ( strcmp( $_GET['settings-updated'], "true" ) == 0 ) {

			/** Render "Settings Saved" message. */
			UI::get_instance()->render_snackbar( esc_html__( 'Settings saved!', 'voicer' ) );

		}

		if ( ! isset( $_GET['tab'] ) ) { return; }

		if ( strcmp( $_GET['tab'], "activation" ) == 0 ) {

			if ( PluginActivation::get_instance()->is_activated() ) {

				/** Render "Activation success" message. */
				UI::get_instance()->render_snackbar( esc_html__( 'Plugin activated successfully.', 'voicer' ), 'success', 5500 );

			} else {

				/** Render "Activation failed" message. */
				UI::get_instance()->render_snackbar( esc_html__( 'Invalid purchase code.', 'voicer' ), 'error', 5500 );

			}

		}

	}

	/**
	 * Get plugin settings with default values.
	 *
	 * @since 2.0.0
	 * @access public
	 * @return void
	 **/
	public function get_options() {

		/** Options. */
		$options = get_option( 'mdp_voicer_settings' );

		/** Design tab settings. */
		$design_settings = get_option( 'mdp_voicer_design_settings' );

		/** Default values. */
		$defaults = [
			'api-key-file'  => '', // API Key file.
			'language'      => 'en-US-Standard-D', // Language.
			'language-code' => 'en-US', // Language Code.
			'audio-profile' => 'handset-class-device', // Audio profile.
			'speaking-rate' => '1', // Speaking rate/speed.
			'pitch'         => '0', // Pitch.

			/** Design Tab. */
			'bgcolor'       => '#0253ee', // Tooltip background color.
			'iconcolor'     => '#fff', // Icon color.
		];

		/** Some checks for key file. Use only JSON file. */
		if ( isset( $options['api-key-file'] ) ) {
			$key_path = get_attached_file( $options['api-key-file'] );
			$ext = pathinfo( $key_path, PATHINFO_EXTENSION );
			if ( 'json' != $ext ) {
				$options['api-key-file'] = '';
			}
        }

		/** Design Tab. */
		if ( isset( $design_settings['bgcolor'] ) ) {
            $options['bgcolor'] = $design_settings['bgcolor'];
		}

		if ( isset( $design_settings['iconcolor'] ) ) {
            $options['iconcolor'] = $design_settings['iconcolor'];
		}

		$results = wp_parse_args( $options, $defaults );

		$this->options = $results;
	}

	/**
	 * Main Settings Instance.
	 *
	 * Insures that only one instance of Settings exists in memory at any one time.
	 *
	 * @static
	 * @return Settings
	 * @since 2.0.0
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Settings ) ) {
			self::$instance = new Settings;
		}

		return self::$instance;
	}

	/**
	 * Throw error on object clone.
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object therefore, we don't want the object to be cloned.
	 *
	 * @return void
	 * @since 2.0.0
	 * @access protected
	 **/
	public function __clone() {
		/** Cloning instances of the class is forbidden. */
		_doing_it_wrong( __FUNCTION__, esc_html__( 'The whole idea of the singleton design pattern is that there is a single object therefore, we don\'t want the object to be cloned.', 'voicer' ), '2.0.2' );
	}

	/**
	 * Disable unserializing of the class.
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object therefore, we don't want the object to be unserialized.
	 *
	 * @return void
	 * @since 2.0.0
	 * @access protected
	 **/
	public function __wakeup() {
		/** Unserializing instances of the class is forbidden. */
		_doing_it_wrong( __FUNCTION__, esc_html__( 'The whole idea of the singleton design pattern is that there is a single object therefore, we don\'t want the object to be unserialized.', 'voicer' ), '2.0.2' );
	}

} // End Class Settings.
